import { Obj } from './Object';
import { Vec3, Mat4 } from 'wtc-math';
import { Drawable } from './Drawable';
declare type CameraOptions = {
    near?: number;
    far?: number;
    fov?: number;
    aspect?: number;
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
    zoom?: number;
};
/**
 * Class representing some Geometry.
 * @extends Obj
 **/
declare class Camera extends Obj {
    /**
     * The near point of the perspective matrix
     * @default .1
     */
    near: number;
    /**
     * The far point of the perspective matrix
     * @default 100
     */
    far: number;
    /**
     * The field of view of the perspective matrix, in degrees
     * @default 45
     */
    fov: number;
    /**
     * The aspect ratio of the perspective matric - normally defined as width / height
     * @default 1
     */
    aspect: number;
    /**
     * The left plane of the orthagraphic view
     */
    left: number;
    /**
     * The right plane of the orthagraphic view
     */
    right: number;
    /**
     * The top plane of the orthagraphic view
     */
    top: number;
    /**
     * The bottom plane of the orthagraphic view
     */
    bottom: number;
    /**
     * The zoom level of the orthagraphic view
     * @default 1
     */
    zoom: number;
    /**
     * The camera projection matrix gives a vector space projection to a subspace.
     */
    projectionMatrix: Mat4;
    /**
     * The camera view matrix transforms vertices from world-space to view-space.
     */
    viewMatrix: Mat4;
    /**
     * The combined projection and view matrix.
     */
    projectionViewMatrix: Mat4;
    /**
     * The position in world space
     */
    worldPosition: Vec3;
    /**
     * The camera type - perspective or orthographic. If left / right are provided this will default to orthographic, otherwise it will default to perspective
     */
    type: 'orthographic' | 'perspective';
    frustum: {
        [key: string]: {
            pos: Vec3;
            constant: number;
        };
    };
    /**
     * Create a camera object
     * @param {Object} __namedParameters - The parameters to be used for the camera
     * @param {number} near - The near point of the perspective matrix
     * @param {number} far - The far point of the perspective matrix
     * @param {number} fov - The fov of the camera, in degrees
     * @param {number} aspect - The camera aspect ratio
     * @param {number} left - The left point of the orthographic camera
     * @param {number} right - The right point of the orthographic camera
     * @param {number} top - The top point of the orthographic camera
     * @param {number} bottom - The bottom point of the orthographic camera
     * @param {number} zoom - The zoom level of the orthographic camera
     */
    constructor({ near, far, fov, aspect, left, right, top, bottom, zoom }?: CameraOptions);
    /**
     * Calculate the parameters necessary for a perspective camera
     * @param {Object} __namedParameters - The parameters to be used for the camera
     * @param {number} near - The near point of the perspective matrix
     * @param {number} far - The far point of the perspective matrix
     * @param {number} fov - The fov of the camera, in degrees
     * @param {number} aspect - The camera aspect ratio
     */
    perspective({ near, far, fov, aspect }?: {
        near?: number;
        far?: number;
        fov?: number;
        aspect?: number;
    }): Camera;
    /**
     * Calculate the parameters necessary for an orthographic camera
     * @param {Object} __namedParameters - The parameters to be used for the camera
     * @param {number} near - The near point of the perspective matrix
     * @param {number} far - The far point of the perspective matrix
     * @param {number} left - The left point of the orthographic camera
     * @param {number} right - The right point of the orthographic camera
     * @param {number} top - The top point of the orthographic camera
     * @param {number} bottom - The bottom point of the orthographic camera
     * @param {number} zoom - The zoom level of the orthographic camera
     */
    orthographic({ near, far, left, right, bottom, top, zoom }?: {
        near?: number;
        far?: number;
        left?: number;
        right?: number;
        bottom?: number;
        top?: number;
        zoom?: number;
    }): Camera;
    /**
     * Update the world view matrices
     * @chainable
     * @returns The camera operated on to make function chainable
     */
    updateMatrixWorld(): Camera;
    /**
     * Look at a position
     * @param {Vec3} target - The position to look at
     * @chainable
     * @returns The camera operated on to make function chainable
     */
    lookAt(target: Vec3): Camera;
    /**
     * Update the frustum parameters based on the prejection view matrix
     */
    updateFrustum(): void;
    /**
     * Determines whether the camera frustum intersects the supplied drawable object. Used mainly for frustum culling.
     * @param {Drawable} node - The node to test intersection against
     * @returns Boolean indicating intersection
     */
    frustumIntersects(node: Drawable): boolean;
    /**
     * Determines the largist axis scale of a matrix
     * @param {Mat4} m - The matrix to find the max axis of
     * @returns The max axis scale
     */
    maxAxisScale(m: Mat4): number;
    /**
     * Determines whether the frustum intersects a sphere
     * @param {Vec3} center - The center of the sphere to test.
     * @param {number} radius - The radius of the sphere to test.
     * @returns Boolean indicating intersection
     */
    frustumIntersectsSphere(center: Vec3, radius: number): boolean;
}
export { Camera, CameraOptions };
