import { WTCGLRenderingContext } from '../types';
import { Mat3, Mat4 } from 'wtc-math';
import { Drawable } from './Drawable';
import { Geometry } from '../geometry/Geometry';
import { Program } from './Program';
import { Camera } from './Camera';
declare type MeshCallback = ({ mesh: Mesh, camera: Camera }: {
    mesh: any;
    camera: any;
}) => void;
/**
 * Class representing a mesh. A mesh is a binding point between geometry and a program.
 * @extends Drawable
 **/
declare class Mesh extends Drawable {
    /**
     * The geometry to render.
     */
    geometry: Geometry;
    /**
     * The mode to use to draw this mesh. Can be one of:
     * - gl.POINTS: Draws a single dot for each vertex
     * - gl.LINE_STRIP: Draws a straight line between the vertices
     * - gl.LINE_LOOP: As above, but loops back.
     * - gl.LINES: Draws many lines between each vertex pair
     * - gl.TRIANGLE_STRIP: Draws a series of triangles between each vertice trio
     * - gl.TRIANGLE_FAN: Draws a series of triangles between each vertice trio, treating the first vertex as the origin of each triangle
     * - gl.TRIANGLES: Draws a triangle for a group of three vertices
     */
    mode: GLenum;
    /**
     * The world matrix projected by the camera's view matrix
     */
    modelViewMatrix: Mat4;
    /**
     * The model-view normal matrix
     */
    normalMatrix: Mat3;
    /**
     * Any callbacks to run before render
     */
    beforeRenderCallbacks: {
        ({ mesh: Mesh, camera: number }: {
            mesh: any;
            camera: any;
        }): void;
    }[];
    /**
     * Any callbacks to run after render
     */
    afterRenderCallbacks: {
        ({ mesh: Mesh, camera: number }: {
            mesh: any;
            camera: any;
        }): void;
    }[];
    /**
     * Create a mesh
     * @param {WTCGLRenderingContext} gl - The WTCGL Rendering context
     * @param {Object} __namedParameters - The parameters for the attribute
     * @param {Geometry} geometry - The geometry for the mesh to render
     * @param {Program} program - The program - shaders and uniforms - used to render the geometry
     * @param {GLenum} mode - The mode to render using
     * @param {boolean} frustumCulled - Whether to apply culling to this object
     * @param {number} renderOrder - The explicit render order of the object. If this is zero, then it will be instead calculated at render time.
     */
    constructor(gl: WTCGLRenderingContext, { geometry, program, mode, frustumCulled, renderOrder }?: {
        geometry?: Geometry;
        program?: Program;
        mode?: GLenum;
        frustumCulled?: boolean;
        renderOrder?: number;
    });
    /**
     * Add a before render callback.
     * @param {function} f - The function to call before render. Expected shape ({ mesh: this, camera })=>void.
     **/
    addBeforeRender(f: MeshCallback): this;
    /**
     * Remove a before render callback.
     * @param {function} f - The function to call before render. Expected shape ({ mesh: this, camera })=>void.
     **/
    removeBeforeRender(f: MeshCallback): void;
    /**
     * Remove all before render callbacks
     */
    removeAllBeforeRender(): void;
    /**
     * Add an after render callback.
     * @param {function} f - The function to call before render. Expected shape ({ mesh: this, camera })=>void.
     **/
    addAfterRender(f: MeshCallback): this;
    /**
     * Remove an after render callback.
     * @param {function} f - The function to call before render. Expected shape ({ mesh: this, camera })=>void.
     **/
    removeAfterRender(f: MeshCallback): void;
    /**
     * Remove all afyer render callbacks
     */
    removeAllAfterRender(): void;
    /**
     * Drw the mesh. If a camera is supplied to the draw call, its various matrices will be added to the program uniforms
     * @param {Camera} camera - The camera to use to supply transformation matrices
     */
    draw({ camera }?: {
        camera?: Camera;
    }): void;
}
export { Mesh };
