import { WTCGLRenderingContext, WTCGLBlendFunction, WTCGLBlendEquation, WTCGLUniformMap, WTCGLAttributeMap } from '../types';
import { Uniform } from './Uniform';
interface WTCGLUniformArray {
    [index: string]: Uniform;
}
/**
 * The program class prvides the rendering setup, internal logic, and state for rendering an object.
 */
export declare class Program {
    /**
     * The ID of the program. Simple auto-incremening. Used for identifying the program for setup.
     */
    id: number;
    /**
     * The WTCGL rendering context object
     */
    gl: WTCGLRenderingContext;
    /**
     * The array uf uniforms for this program
     */
    uniforms: WTCGLUniformArray;
    /**
     * Whether to render transparency.
     * @default false
     */
    transparent: boolean;
    /**
     * Whether to depth test objects in this program
     * @default true
     */
    depthTest: boolean;
    /**
     * Whether to write depth information
     * @default true
     */
    depthWrite: boolean;
    /**
     * Face culling used.
     * @default gl.BACK
     */
    cullFace: GLenum;
    /**
     * How to determine if a face is front-facing, whether it's points are drawn clockwise or counter-clockwise. Default is counter-clockwise.
     * @default gl.CCW
     */
    frontFace: GLenum;
    /**
     * The depth function to use when determinging the current pixel against the depth buffer
     * @default gl.LESS
     */
    depthFunc: GLenum;
    /**
     * The blend function to use
     */
    blendFunc: WTCGLBlendFunction;
    /**
     * The blend equation to use
     */
    blendEquation: WTCGLBlendEquation;
    /**
     * The webgl program store
     */
    program: WebGLProgram;
    /**
     * A map of uniform locations in use within the program shaders
     */
    uniformLocations: WTCGLUniformMap;
    /**
     * A map of attribute locations in use within the program shaders
     */
    attributeLocations: WTCGLAttributeMap;
    /**
     * A join of the found attributes. Used for addressing vertex array objects on the geometry.
     */
    attributeOrder: string;
    /**
     * The texture unit. Used for addressing texture units in-program.
     */
    textureUnit: number;
    /**
     * Create a Program
     * @param gl - The WTCGL Rendering context
     * @param __namedParameters - The parameters for the Program
     * @param vertex - The vertext shader
     * @param fragment - The fragment shader
     * @param uniforms - An object of uniforms for use in the program
     * @param transparent - Whether to render the program with transparency
     * @param cullFace - What method to use for determining face culling
     * @param frontFace - What method to use for determining the front of a face
     * @param depthTest - Whether to test the depth of a fragment against the depth buffer
     * @param depthWrite - Whether to write the depth information
     * @param depthFunc - The function to use when depth testing
     */
    constructor(gl: WTCGLRenderingContext, { vertex, fragment, uniforms, transparent, cullFace, frontFace, depthTest, depthWrite, depthFunc, transformFeedbackVaryings }?: {
        vertex?: string;
        fragment?: string;
        uniforms?: WTCGLUniformArray;
        transparent?: boolean;
        cullFace?: GLenum;
        frontFace?: GLenum;
        depthTest?: boolean;
        depthWrite?: boolean;
        depthFunc?: GLenum;
        transformFeedbackVaryings?: string[];
    });
    /**
     * Set the blend function based on source and destination parameters
     * @param src - the source blend function
     * @param dst - The destination blend function
     * @param srcAlpha - the source blend function for alpha blending
     * @param dstAlpha - the destination blend function for alpha blending
     */
    setBlendFunc(src: GLenum, dst: GLenum, srcAlpha?: GLenum, dstAlpha?: GLenum): void;
    /**
     * set the blend equation
     * @param modeRGB - The mode to blend when using RGB
     * @param modeAlpha - The mode to blend when using RGBA
     */
    setBlendEquation(modeRGB: GLenum, modeAlpha: GLenum): void;
    /**
     * Apply the program state object to the renderer
     */
    applyState(): void;
    /**
     * Set up the program for use
     * @param flipFaces - Flip the faces, for when a mesh is scaled in teh negative
     */
    use({ flipFaces }?: {
        flipFaces?: boolean;
    }): void;
    /**
     * Delete the program
     */
    remove(): void;
}
export {};
