import { WTCGLRenderingContext } from '../types';
import { Texture } from './Texture';
/**
 * Create a render target. A render target allows you to render a scene to a texture, instead of to screen. And can be used to either render a different view for composition to a scene, or to create advanced post processing effects.
 */
declare class RenderTarget {
    /**
     * The WTCGL rendering context.
     */
    gl: WTCGLRenderingContext;
    /**
     * The WebGL frame buffer object to render to.
     */
    buffer: WebGLFramebuffer;
    /**
     * The texture array. If you supply only one colour, you can just output this as you normally would with gl_FragColor, otherwise you need to output one at a time with gl_FragData[x]
     */
    textures: Texture[];
    /**
     * The depth buffer.
     */
    depthBuffer: WebGLFramebuffer;
    /**
     * The stencil buffer.
     */
    stencilBuffer: WebGLFramebuffer;
    /**
     * The depth-stencil buffer.
     */
    depthStencilBuffer: WebGLFramebuffer;
    /**
     * The depth texture.
     */
    depthTexture: Texture | null;
    /**
     * The width of the target.
     */
    width: number;
    /**
     * The height of the target.
     */
    height: number;
    /**
     * Whether to render a depth buffer.
     */
    depth: boolean;
    /**
     * Whether to render a stencil buffer.
     */
    stencil: boolean;
    /**
     * A GLEnum representing the binding point for the texture / buffer.
     * @default gl.FRAMEBUFFER
     */
    target: GLenum;
    /**
     * Create a render target object.
     * @param {WTCGLRenderingContext} gl - The WTCGL Rendering context
     * @param __namedParameters - The parameters to initialise the renderer.
     * @param width - The width of the render target.
     * @param height - The height of the render target.
     * @param target - The binding point for the frame buffers.
     * @param colour - The number of colour attachments to create.
     * @param depth - Whether to create a depth buffer
     * @param stencil - Whether to create a stencil buffer
     * @param wrapS - Wrapping
     * @param wrapT - Wrapping
     * @param minFilter - The filter to use when rendering smaller
     * @param magFilter - The filter to use when enlarging
     * @param type - The texture type. Typically one of gl.UNSIGNED_BYTE, gl.FLOAT, ext.HALF_FLOAT_OES
     * @param format - The texture format.
     * @param internalFormat - the texture internalFormat.
     * @param unpackAlignment - The unpack alignment for pixel stores.
     * @param premultiplyAlpha - Whether to use premultiplied alpha for stored textures.
     */
    constructor(gl: WTCGLRenderingContext, { data, width, height, target, colour, depth, stencil, wrapS, wrapT, minFilter, magFilter, type, format, internalFormat, unpackAlignment, premultiplyAlpha }?: {
        data?: Float32Array | null;
        width?: number;
        height?: number;
        target?: GLenum;
        colour?: number;
        depth?: boolean;
        stencil?: boolean;
        depthTexture?: Texture | null | boolean;
        wrapS?: GLenum;
        wrapT?: GLenum;
        minFilter?: GLenum;
        magFilter?: GLenum;
        type?: GLenum;
        format?: GLenum;
        internalFormat?: GLenum;
        unpackAlignment?: 1 | 2 | 4 | 8;
        premultiplyAlpha?: boolean;
    });
    /**
     * Returns the first texture, for the majority os use cases.
     */
    get texture(): Texture;
}
export { RenderTarget };
