import { Vec2 } from 'wtc-math';
import { WTCGLRendererState, WTCGLRenderingContext, WTCGLExtensions, WTCGLRendererParams } from '../types';
import { Camera } from './Camera';
import { RenderTarget } from './RenderTarget';
import { Obj } from './Object';
import { Drawable } from './Drawable';
/**
 * Create a renderer. This is responsible for bringing together the whole state and, eventually, rendering to screen.
 */
declare class Renderer {
    #private;
    /**
     * The pixel aspect ratio of the renderer.
     * @default window.devicePixelRatio or 2, whichever is smaller.
     */
    dpr: number;
    /**
     * Whether to render an alpha channel. This property is passed to the rendering context.
     * @default false
     */
    alpha: boolean;
    /**
     * Whether to clear the color bit
     * @default true
     */
    colour: boolean;
    /**
     * Whether to render a depth buffer. This property is passed to the rendering context.
     * @default true
     */
    depth: boolean;
    /**
     * Whether to render a stencil buffer. This property is passed to the rendering context.
     * @default false
     */
    stencil: boolean;
    /**
     * Whether to use premultiplied alphs. This property is passed to the rendering context.
     * @default false
     */
    premultipliedAlpha: boolean;
    /**
     * Whether to automatically clear the buffers before render.
     * @default true
     */
    autoClear: boolean;
    /**
     * Whether the context we've retrieved is webGL2 or not.
     */
    isWebgl2: boolean;
    /**
     * The WTCGL rendering context.
     */
    gl: WTCGLRenderingContext;
    /**
     * The rendering state. Allows us to avoid redundant calls on methods used internally
     */
    state: WTCGLRendererState;
    /**
     * Stores the enabled extensions
     */
    extensions: WTCGLExtensions;
    /**
     * Stored device parameters such as max allowable units etc.
     */
    parameters: WTCGLRendererParams;
    /**
     * Stores the current geometry being rendered. Used to otimise geo rendering.
     */
    currentGeometry: string;
    /**
     * The WebGL2RenderingContext.vertexAttribDivisor() method of the WebGL 2 API modifies the rate at which generic vertex attributes advance when rendering multiple instances of primitives with
     */
    vertexAttribDivisor: (index: number, divisor: number) => void;
    /**
     * The WebGL2RenderingContext.drawArraysInstanced() method of the WebGL 2 API renders primitives from array data like the gl.drawArrays() method. In addition, it can execute multiple instances of the range of elements.
     */
    drawArraysInstanced: (mode: GLenum, first: number, count: number, instanceCound: number) => void;
    /**
     * The WebGL2RenderingContext.drawElementsInstanced() method of the WebGL 2 API renders primitives from array data like the gl.drawElements() method. In addition, it can execute multiple instances of a set of elements.
     */
    drawElementsInstanced: (mode: GLenum, count: number, type: GLenum, offset: GLintptr, instanceCount: number) => void;
    /**
     * The WebGL2RenderingContext.createVertexArray() method of the WebGL 2 API creates and initializes a WebGLVertexArrayObject object that represents a vertex array object (VAO) pointing to vertex array data and which provides names for different sets of vertex data.
     */
    createVertexArray: () => WebGLVertexArrayObject;
    /**
     * The WebGL2RenderingContext.bindVertexArray() method of the WebGL 2 API binds a passed WebGLVertexArrayObject object to the buffer.
     */
    bindVertexArray: (vertexArray: WebGLVertexArrayObject) => void;
    /**
     * The WebGL2RenderingContext.deleteVertexArray() method of the WebGL 2 API deletes a given WebGLVertexArrayObject object.
     */
    deleteVertexArray: (vertexArray: WebGLVertexArrayObject) => void;
    /**
     * The WebGL2RenderingContext.drawBuffers() method of the WebGL 2 API defines draw buffers to which fragment colors are written into. The draw buffer settings are part of the state of the currently bound framebuffer or the drawingbuffer if no framebuffer is bound.
     */
    drawBuffers: (buffers: GLenum[]) => void;
    currentProgram: number;
    /**
     * create a renderer
     * @param __namedParameters - The parameters to initialise the renderer.
     * @param canvas - The canvas HTML element to render to.
     * @param width - The width of the canvas.
     * @param height - The height of the canvas.
     * @param dpr - The pixel aspect ratio of the canvas.
     * @param alpha - Whether to render an alpha channel.
     * @param depth - Whether to render a depth buffer.
     * @param stencil - Whether to render a stencil buffer.
     * @param premultipliedAlpha - Whether to use premultiplied alpha.
     * @param preserveDrawingBuffer - Preserve the drawing buffer between calls. Useful for when downloading the canvas to an image.
     * @param powerPreference - WebGL power preference.
     * @param autoClear - Whether to clear the canvas between draw.
     * @param webgl - The webGL version to try to use - 1, or 2
     */
    constructor({ canvas, width, height, dpr, alpha, depth, stencil, antialias, premultipliedAlpha, preserveDrawingBuffer, powerPreference, autoClear, webgl }?: {
        canvas?: HTMLCanvasElement;
        width?: number;
        height?: number;
        dpr?: number;
        alpha?: boolean;
        depth?: boolean;
        stencil?: boolean;
        antialias?: boolean;
        premultipliedAlpha?: boolean;
        preserveDrawingBuffer?: boolean;
        powerPreference?: string;
        autoClear?: boolean;
        webgl?: number;
    });
    set dimensions(v: Vec2);
    get dimensions(): Vec2;
    /**
     * Set up the viewport for rendering.
     * @param dimensions - The dimensions of the viewport
     * @param position - The position of the viewport
     */
    setViewport(dimensions: Vec2, position: Vec2): void;
    /**
     * Enables specific WebGL capabilities for this context.
     * See [MDN](https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/enable) for more details
     * @param id - The ID of the capability to enable
     */
    enable(id: GLenum): void;
    /**
     * Disables specific WebGL capabilities for this context.
     * @param id - The ID of the capability to enable
     */
    disable(id: GLenum): void;
    /**
     * Set's various blend functions are used for blending pixel calculations
     * See [MDN](https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/blendFunc) for more details
     * If alpha functions are provided, this will call gl.blendFuncSeparate, otherwise blendFunc
     * @param src - A WebGL_API.Types specifying a multiplier for the RGB source blending factors.
     * @param dst - A WebGL_API.Types specifying a multiplier for the RGB destination blending factors.
     * @param srcAlpha - A WebGL_API.Types specifying a multiplier for the alpha source blending factor.
     * @param dstAlpha - A WebGL_API.Types specifying a multiplier for the alpha destination blending factor.
     */
    setBlendFunc(src: GLenum, dst: GLenum, srcAlpha: GLenum, dstAlpha: GLenum): void;
    /**
     * Sets a blending function for use in the application.
     * See [MDN](https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/blendEquation) for more information
     * @param modeRGB - The function to be used in RGB models
     * @param modeAlpha - The functions to be used for both RGB and alpha models
     */
    setBlendEquation(modeRGB: GLenum, modeAlpha: GLenum): void;
    /**
     * Sets the cull face bit
     */
    set cullFace(value: GLenum);
    get cullFace(): GLenum;
    /**
     * Sets the front face bit
     */
    set frontFace(value: GLenum);
    get frontFace(): GLenum;
    /**
     * Sets the depth mask bit
     */
    set depthMask(value: boolean);
    get depthMask(): boolean;
    /**
     * Sets the depth function bit
     */
    set depthFunc(value: GLenum);
    get depthFunc(): GLenum;
    /**
     * Sets the active texture value
     */
    set activeTexture(value: number);
    get activeTexture(): number;
    /**
     * Binds a given WebGLFramebuffer to a target .
     * @param __namedParameters
     * @param target - A GLenum specifying the binding point (target). Typically only `gl.FRAMEBUFFER` see [MDN](https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/bindFramebuffer) for more information
     * @param buffer - A WebGLFramebuffer object to bind. If framebuffer is null, then the canvas (which has no WebGLFramebuffer object) is bound.
     * @returns
     */
    bindFramebuffer({ target, buffer }?: {
        target?: GLenum;
        buffer?: WebGLFramebuffer | null;
    }): void;
    /**
     * Finds and enables a webGL extension and, if it has a corresponding function, returns that.
     * @param extension - The extension identifier.
     * @param webgl2Func -The name of the webGL2 function to return.
     * @param extFunc - The name of the webGL1 functiont to return.
     * @returns - A WebGL function, bound to this renderer or null (if no function exists)
     */
    getExtension(extension: string, webgl2Func?: string, extFunc?: string): any;
    /**
     * An array sort for opaque elements
     * @param a - A renderable object for sorting
     * @param b - A renderable object for sorting
     * @returns The number to determine relative position
     */
    sortOpaque(a: Drawable, b: Drawable): number;
    /**
     * An array sort for transparent elements
     * @param a - A renderable object for sorting
     * @param b - A renderable object for sorting
     * @returns The number to determine relative position
     */
    sortTransparent(a: Drawable, b: Drawable): number;
    /**
     * An array sort for UI (no depth) elements
     * @param a - A renderable object for sorting
     * @param b - A renderable object for sorting
     * @returns The number to determine relative position
     */
    sortUI(a: Drawable, b: Drawable): number;
    /**
     * Retrieves the list of renderable objects sorted by position, explicit render order and applied depth
     * @param __namedParameters
     * @param scene - The root scene to render.
     * @param camera - The camera to use to determine the render list, applies frustum culling etc.
     * @param frustumCull - Whether to apply frustum culling
     * @param sort - Whether to sort the list at all.
     * @returns - An array of renderable objects
     */
    getRenderList({ scene, camera, frustumCull, sort }: {
        scene: Obj;
        camera: Camera;
        frustumCull: boolean;
        sort: boolean;
    }): Drawable[];
    /**
     * Renders a scene
     * @param __namedParameters
     * @param scene - The renderable object to render.
     * @param camera - The camera to render with. If not supplied will just render as is.
     * @param target - The render target to render to.
     * @param update - Whether to update all of the object worl matrices prior to rendering.
     * @param sort - Whether to sort the objects prior to rendering.
     * @param frustumCull - Whether to apply frustum culling prior to rendering.
     * @param clear - Whether to clear the scene prior to rendering. Only matters if renderer.autoClear is false.
     */
    render({ scene, camera, target, update, sort, frustumCull, clear, viewport }: {
        scene: Obj;
        camera?: Camera;
        target?: RenderTarget | null;
        update?: boolean;
        sort?: boolean;
        frustumCull?: boolean;
        clear?: boolean;
        viewport?: [Vec2, Vec2];
    }): void;
}
export { Renderer };
