import { WTCGLRendererState, WTCGLTextureState, WTCGLRenderingContext } from '../types';
/**
 * A texture class contains image data for use in a shader. Along with the image data, the texture contains state variable that determine how secondary conditions, like wrapping and interpolation work.
 */
declare class Texture {
    /**
     * A unique ID for the texture object, allows us to determine whether a texture has already been bound or not.
     */
    id: number;
    /**
     * The WTCGL rendering context.
     */
    gl: WTCGLRenderingContext;
    /**
     * The image element representing the texture. Can be an HTML image, video, or canvas
     */
    image: HTMLImageElement | HTMLVideoElement | HTMLCanvasElement;
    data: Float32Array | null;
    /**
     * The WebGL texture object containing the data and state for WebGL
     */
    texture: WebGLTexture;
    /**
     * The width of the texture.
     */
    width: number;
    /**
     * The height of the texture.
     */
    height: number;
    /**
     * The binding point for the texture.
     * @default gl.TEXTURE_2D
     */
    target: GLenum;
    /**
     * The texture type. Typically one of gl.UNSIGNED_BYTE, gl.FLOAT, ext.HALF_FLOAT_OES
     * @default gl.UNSIGNED_BYTE
     */
    type: GLenum;
    /**
     * The texture format.
     * @default gl.RGBA
     */
    format: GLenum;
    /**
     * the texture internal format.
     * @default gl.RGBA
     */
    internalFormat: GLenum;
    /**
     * The filter to use when rendering smaller.
     */
    minFilter: GLenum;
    /**
     * The filter to use when rendering larger.
     */
    magFilter: GLenum;
    /**
     * Wrapping. Normally one of gl.CLAMP_TO_EDGE, gl.REPEAT, gl.MIRRORED REPEAT
     * @default gl.CLAMP_TO_EDGE
     */
    wrapS: GLenum;
    /**
     * Wrapping. Normally one of gl.CLAMP_TO_EDGE, gl.REPEAT, gl.MIRRORED REPEAT
     * @default gl.CLAMP_TO_EDGE
     */
    wrapT: GLenum;
    /**
     * Whether to generate mip maps for this texture. This requires that the texture be power of 2 in size
     * @default true
     */
    generateMipmaps: boolean;
    /**
     * Whether the texture is interpolated as having premultiplied alpha.
     * @default false
     */
    premultiplyAlpha: boolean;
    /**
     * Whether to flip the Y direction of the texture
     */
    flipY: boolean;
    /**
     * The unpack alignment for pixel stores.
     */
    unpackAlignment: 1 | 2 | 4 | 8;
    /**
     * The anistropic filtering level for the texture.
     * @default 0
     */
    anisotropy: number;
    /**
     * A GLint specifying the level of detail. Level 0 is the base image level and level n is the nth mipmap reduction level. Only relevant if we have mipmaps.
     * @default 0
     */
    level: number;
    /**
     * The image store
     */
    store: {
        image: HTMLImageElement | HTMLVideoElement | HTMLCanvasElement;
    };
    /**
     * A local reference to the renderer state
     */
    glState: WTCGLRendererState;
    /**
     * Texture state, contains the basic texture properties.
     */
    state: WTCGLTextureState;
    /**
     * Whether the texture needs an update in memory
     */
    needsUpdate: boolean;
    /**
     * Create a render target object.
     * @param {WTCGLRenderingContext} gl - The WTCGL Rendering context
     * @param __namedParameters - The parameters to initialise the renderer.
     * @param target - The binding point for the frame buffers.
     * @param type - The texture type. Typically one of gl.UNSIGNED_BYTE, gl.FLOAT, ext.HALF_FLOAT_OES
     * @param format - The texture format.
     * @param internalFormat - the texture internalFormat.
     * @param wrapS - Wrapping
     * @param wrapT - Wrapping
     * @param generateMipmaps - Whether to generate mips for this texture
     * @param minFilter - The filter to use when rendering smaller
     * @param magFilter - The filter to use when enlarging
     * @param premultiplyAlpha - Whether to use premultiplied alpha for stored textures.
     * @param flipY - Whether to flip the Y component of the supplied image
     * @param anistropy - The anistropic filtering level for the texture.
     * @param unpackAlignment - The unpack alignment for pixel stores.
     * @param level - A GLint specifying the level of detail. Level 0 is the base image level and level n is the nth mipmap reduction level. Only relevant if we have mipmaps.
     * @param width - The width of the render target.
     * @param height - The height of the render target.
     */
    constructor(gl: WTCGLRenderingContext, { image, data, target, type, format, internalFormat, wrapS, wrapT, generateMipmaps, minFilter, magFilter, premultiplyAlpha, unpackAlignment, flipY, anisotropy, level, width, // used for RenderTargets or Data Textures
    height }?: {
        image?: HTMLImageElement | HTMLVideoElement | HTMLCanvasElement;
        data?: Float32Array | null;
        target?: GLenum;
        type?: GLenum;
        format?: GLenum;
        internalFormat?: GLenum;
        wrapS?: GLenum;
        wrapT?: GLenum;
        generateMipmaps?: boolean;
        minFilter?: GLenum;
        magFilter?: GLenum;
        premultiplyAlpha?: boolean;
        unpackAlignment?: 1 | 2 | 4 | 8;
        flipY?: boolean;
        anisotropy?: number;
        level?: number;
        width?: number;
        height?: number;
    });
    /**
     * Bind the texture. Skips over if it's determined that the texture is already bound.
     */
    bind(): void;
    /**
     * Update the texture in graphics memory to the internal image and perform various state updates on an as-needs basis.
     * @param textureUnit The texture unit to update against.
     * @returns
     */
    update(textureUnit?: number): void;
}
export { Texture };
