import { WTCGLRendererState, WTCGLRenderingContext, WTCGLGeometryAttributeCollection, WTCGLGeometryAttribute, WTCGLBounds } from '../types';
import { Program } from '../core/Program';
import { TransformFeedback } from '../core/TransformFeedback';
/**
 * Class representing some Geometry.
 **/
export declare class Geometry {
    /**
     * The unique ID of the Geometry.
     */
    id: number;
    /**
     * The WTCGL rendering context.
     */
    gl: WTCGLRenderingContext;
    /**
     * The WTCGL attribute collection that describes this geometry.
     */
    attributes: WTCGLGeometryAttributeCollection;
    /**
     * An array of vertex array objects that represent the different attributes.
     */
    VAOs: {
        [key: string]: WebGLVertexArrayObject;
    };
    /**
     * Any supplied transform feedback objects
     */
    transformFeedbacks: TransformFeedback;
    transformFeedbackIndex: number;
    /**
     * The range of supplied elements to draw.
     */
    drawRange: {
        start: number;
        count: number;
    };
    /**
     * The number of instances to draw.
     */
    instancedCount: number;
    /**
     * A boolean indicating whether the geometry is an instanced geometry or not.
     */
    isInstanced: boolean;
    /**
     * An object defining the boundary of the geometry
     */
    bounds: WTCGLBounds;
    /**
     * An object defining all of the rendering state properties.
     */
    glState: WTCGLRendererState;
    /**
     * Create a geometry object.
     * @param {WTCGLRenderingContext} gl - The WTCGL Rendering context
     * @param {WTCGLGeometryAttributeCollection} attributes - A collection of attributes for the geometry. Typically includes vertex positions in 2 or 3d, normals etc.
     */
    constructor(gl: WTCGLRenderingContext, attributes?: WTCGLGeometryAttributeCollection, transformFeedbacks?: TransformFeedback);
    /**
     * Add an attribute
     * @param {string} key - The key for the attribute
     * @param {WTCGLGeometryAttribute} attr - The Geometry attribute, basically the data to be injected
     */
    addAttribute(key: string, attr: WTCGLGeometryAttribute): void;
    /**
     * Sets up the draw range, used to determined which properties are drawn
     * @param {number} start - The start of the range
     * @param {number} count - The number of properties to draw
     */
    setDrawRange(start: number, count: number): void;
    /**
     * Sets up the number of instances to draw
     * @param {number} value - The number of geometry instances to draw
     */
    setInstancedCount(value: number): void;
    /**
     * Create a new vertex array object bind Attributes to it
     * @param {Program} program - The program to use to determine attribute locations
     */
    createVAO(program: Program): void;
    /**
     * Binds all attributes as derived from the program to attribute objects supplied to the geometry
     * @param {Program} program - The program to use to determine attribute locations
     */
    bindAttributes(program: Program): void;
    bindTransformFeedbacks(): void;
    /**
     * Draw the geometry
     * @param {Object} __namedParameters - The parameters to be used for the draw command
     * @param {Program} program - The Program object to draw to
     * @param {GLenum} mode - The mode to draw the object in
     */
    draw({ program, mode }: {
        program?: Program;
        mode: GLenum;
    }): void;
    /**
     * Returns the position atribute array
     * @returns {WTCGLGeometryAttribute}
     */
    getPosition(): WTCGLGeometryAttribute;
    /**
     * Computes the bounding box of the geometry. If no attribute is provided to compue with, try to use the position attribute array by default.
     * @param {WTCGLGeometryAttribute} attr - The attribute array to compute the bounding box off
     */
    computeBoundingBox(attr: WTCGLGeometryAttribute | null): void;
    /**
     * Computes the bounding sphere of the geometry. If no attribute is provided to compue with, try to use the position attribute array by default.
     * @param {WTCGLGeometryAttribute} attr - The attribute array to compute the bounding box off
     */
    computeBoundingSphere(attr?: WTCGLGeometryAttribute | null): void;
    /**
     * Remoce all of the vertex array objects and buffers from memory
     */
    remove(): void;
}
